@@ -763,7 +763,8 @@ static void on_find_clicked(GtkButton * button, gpointer data)
 	    if(strcmp(s, _(ftypes[j])) == 0)
 	    {
 		argument[i++] = "-type";
-		argument[i++] = find_ft[j];
+		if (argument[0] == FIND && strcmp("reg",find_ft[j])==0) argument[i++] = "file";
+		else argument[i++] = find_ft[j];
 		break;
 	    }
 	}
