@@ -92,7 +92,7 @@ autotype_t autotype[]= {
 	{".tgz","tar -xzf ",N_("Extract files from"),N_("Extract into"),1},
 	{".tgz","tar -tzf ","Tarball contents",NULL,1}, /* XXX remove when tarball expansion enabled */
 #if 1
-	{".iso","burncd -f /dev/acd0c -s max data ","burncd",NULL,1},
+	{".iso","burncd -f /dev/acd0c -s max data \%s fixate","burncd",NULL,1},
 	{".iso","cdrecord -v speed=24 dev=0,0 ","cdrecord",NULL,1},
 #endif
 	
@@ -132,40 +132,18 @@ autotype_t autotype[]= {
 	{".deb","dpkg -i ",N_("Install package"),NULL,1},
 #endif
 	/* assorted packages (not queued) */
+#if 0
+	/* some of these are now the application mime types 
+	 * remove them from xffm 4.1 when the duplicate types
+	 * is introduced into the applications.xml file */
 	{".lyx","lyx","lyx",NULL,0},
 	{".ps","gv","gv",NULL,0},
 	{".pdf","gv","gv",NULL,0},
-	{".abw","abiword","abiword",NULL,0},
-	{".rtf","abiword","abiword",NULL,0},
-	{".rtf","oowriter","oowriter",NULL,0},
-	{".rtf","swriter","swriter",NULL,0},
 	{".dvi","xdvi","xdvi",NULL,0},
-	{".html","mozilla","mozilla",NULL,0},
-	{".html","dillo","dillo",NULL,0},
-	{".html","netscape","netscape",NULL,0},
 	{".sdw","oowriter","oowriter",NULL,0},
 	{".sdw","swriter","swriter",NULL,0},
-	{".doc","abiword","abiword",NULL,0},
-	{".doc","oowriter","oowriter",NULL,0},
-	{".doc","swriter","swriter",NULL,0},
-	{".log","gvim","gvim",NULL,0},
-	{".tex","gvim","gvim",NULL,0},
-	{".txt","abiword","abiword",NULL,0},
-	{".txt","oowriter","oowriter",NULL,0},
-	{".txt","swriter","swriter",NULL,0},
-	{".txt","emacs","emacs",NULL,0},
-	{".txt","gvim","gvim",NULL,0},
-	{".sdc","oocalc","oocalc",NULL,0},
-	{".sxc","oocalc","oocalc",NULL,0},
-	{".sdc","scalc","scalc",NULL,0},
-	{".sxc","scalc","scalc",NULL,0},
-	{".sh","gvim","gvim",NULL,0},
-	{".py","gvim","gvim",NULL,0},
-	{".bash","gvim","gvim",NULL,0},
-	{".csh","gvim","gvim",NULL,0},
-	{".pl","gvim","gvim",NULL,0},
-	{".wav","soxplay","soxplay",NULL,0},
-
+	{".html","dillo","dillo",NULL,0},
+	{".abw","abiword","abiword",NULL,0},
 	{".bmp","display","display",NULL,0},
 	{".eps","display","display",NULL,0},
 	{".gif","display","display",NULL,0},
@@ -175,6 +153,43 @@ autotype_t autotype[]= {
 	{".jpeg","display","display",NULL,0},
 	{".png","display","display",NULL,0},
 	{".xpm","display","display",NULL,0},
+	{".txt","gvim","gvim",NULL,0},
+	{".sdc","oocalc","oocalc",NULL,0},
+	{".sxc","oocalc","oocalc",NULL,0},
+	{".sdc","scalc","scalc",NULL,0},
+	{".sxc","scalc","scalc",NULL,0},
+	{".wav","soxplay","soxplay",NULL,0},
+
+	
+/* extras entries in menu:
+ * for 4.1 these should be added to the mime application file
+ * and processed into xffm from there... (allowing
+ * repeat entries from the mime application file)
+ * */
+	{".wav","soxplay","soxplay",NULL,0},
+	{".tex","abiword","abiword",NULL,0},
+	{".tex","gvim","gvim",NULL,0},
+	{".doc","abiword","abiword",NULL,0},
+	{".doc","oowriter","oowriter",NULL,0},
+	{".doc","swriter","swriter",NULL,0},
+	{".dvi","xterm -e dvips ","dvips",NULL,0},
+	{".pdf","xterm -e pdf2ps ","pdf2ps",NULL,0},
+	{".rtf","abiword","abiword",NULL,0},
+	{".rtf","oowriter","oowriter",NULL,0},
+	{".rtf","swriter","swriter",NULL,0},
+	{".html","dillo","dillo",NULL,0},
+	{".html","mozilla","mozilla",NULL,0},
+	{".html","netscape","netscape",NULL,0},
+	{".log","gvim","gvim",NULL,0},
+	{".txt","abiword","abiword",NULL,0},
+	{".txt","oowriter","oowriter",NULL,0},
+	{".txt","swriter","swriter",NULL,0},
+	{".txt","emacs","emacs",NULL,0},
+	{".sh","gvim","gvim",NULL,0},
+	{".py","gvim","gvim",NULL,0},
+	{".bash","gvim","gvim",NULL,0},
+	{".csh","gvim","gvim",NULL,0},
+	{".pl","gvim","gvim",NULL,0},
 
 	{".gif","gimp","gimp",NULL,0},
 	{".tiff","gimp","gimp",NULL,0},
@@ -183,6 +198,7 @@ autotype_t autotype[]= {
 	{".jpeg","gimp","gimp",NULL,0},
 	{".png","gimp","gimp",NULL,0},
 	{".xpm","gimp","gimp",NULL,0},
+#endif
 	/*{"","","",NULL},*/
 	{NULL,NULL}
 };
@@ -313,6 +329,7 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
     GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
     tree_details_t *tree_details = (tree_details_t *) get_tree_details(treeview);
+    gchar *cmd=NULL,*text=NULL;
 
 
     if(tree_details->loading)
@@ -342,6 +359,8 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 	  showit(popup, "cut1");
 	  showit(popup, "copy1");
 	  showit(popup, "remove1");
+	  showit(popup, "properties1");
+	  showit(popup, "popup_disk_usage");
  	  gtk_menu_popup(GTK_MENU(popup), NULL, NULL, NULL, NULL, 3, t);
    }
     
@@ -349,6 +368,7 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 	    /* single item selected */
 	    
 	gtk_tree_selection_unselect_all(selection);
+        clear_dnd_selection_list();
 	gtk_tree_selection_select_path(selection, selectpath);
 	gtk_tree_model_get_iter(treemodel, &iter, selectpath);
 	gtk_tree_selection_select_iter(selection, &iter);
@@ -377,6 +397,8 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 	  showit(popup, "separator25");
 	  showit(popup, "separator13");
 	  showit(popup, "select_all2");	
+	        showit(popup, "properties1");
+	        showit(popup, "popup_disk_usage");
 	}
 	if(IS_ROOT_TYPE(en->type)) {
 		which_branch=0;
@@ -441,7 +463,7 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 	    }
 	    if(IS_PATH(en->type))
 	    {
-        	gchar *cmd=NULL;
+		int j;
       		reg_t *prg = reg_prog_by_file (en->path);
 		showit(popup, "file1");
 		showit(popup, "remove1");
@@ -481,7 +503,6 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 			/* look in registered programs first */
 			/* FIXME: should check whether prg->app can be run
 			 * before generating the menu entry... */
-		  gchar *text;
 		  GtkWidget *label,*a=lookup_widget(popup,"autotype_R");
     		  label = gtk_bin_get_child(GTK_BIN(a));
 		  if (prg->arg) {
@@ -492,41 +513,70 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 			  cmd=g_strconcat(prg->app, NULL);
 					  /*" ",FILENAME(en),NULL);*/
 		  }
-		  text=g_strconcat(cmd," ",FILENAME(en),NULL);
+		  text=g_strconcat(cmd," \'",FILENAME(en),"\'",NULL);
 		  gtk_label_set_text((GtkLabel *)label,my_utf_string(text));
 		  g_free(text);text=NULL;
 		  showit(popup, "autotype_R");
-      	  	} 
+      	  	}
+
 		{/* autotypes */	      
       		   char *loc;
-		   int i,j;
+		   int i;
 		   GtkWidget *label;
 		   for (j=i=0;auto_C_name[j]&&autotype[i].extension;i++){
 		       GtkWidget *a=lookup_widget(popup,auto_C_name[j]);
 		       loc=strstr(en->path,autotype[i].extension);
 		       if ((loc)&&(strcmp(loc,autotype[i].extension)==0)){
-		           gchar *text,*prog;
+		           gchar *prog;
     		  	   label = gtk_bin_get_child(GTK_BIN(a));
 		           text=g_strconcat(_(autotype[i].label),
-					   " ", FILENAME(en),
+					   " \"", FILENAME(en),"\"",
 					   NULL);
 			   gtk_label_set_text((GtkLabel *)label,
 					   my_utf_string(text));
 			   g_free(text); text=NULL;
+			   
 			   prog=g_strdup(autotype[i].command);
 			   if (strstr(prog," ")) prog=strtok(prog," ");
 			   text=g_find_program_in_path(prog);
+					   
 		  	   if (text && !(cmd && strcmp(prog,cmd)==0)) {
 				   showit(popup, auto_C_name[j]);
 			   }
+			   			     
 			   g_free(text);text=NULL;
 			   g_free(prog);prog=NULL;
-			   g_free(cmd);cmd=NULL;
 			   j++;
 		       }
 		   }
 		}
-
+	        /* duplicate mime types */
+		{
+		  GtkWidget *label;
+		  int i;
+		  const char *prog;
+		  for (i=0; j<10 && i<10;i++){
+			  /*printf("DBG: duplicate slot %d\n",j);*/
+		       	  GtkWidget *a=lookup_widget(popup,auto_C_name[j]);
+			  prog=reg_duplicate_app(en->path,i);
+			  if (!prog) {
+				  /*printf("DBG: no duplicate!\n");*/
+				  break;
+			  }
+			  /*printf("DBG: duplicate FOUND!\n");*/
+    		  	  label = gtk_bin_get_child(GTK_BIN(a));
+		          text=g_strconcat(prog," \"", FILENAME(en),"\"",NULL);
+			  gtk_label_set_text((GtkLabel *)label,my_utf_string(text));
+				   
+		  	  if (!(cmd && strcmp(prog,cmd)==0)) {
+				   showit(popup, auto_C_name[j]);
+			  } 				   
+			  j++;   
+			  g_free(text);text=NULL;
+		  } 
+		}
+		g_free(cmd);cmd=NULL;		   
+		
 		if(IS_TRASH_TYPE(en->type))
 		{
 		    showit(popup, "uncollect_from_trash");
@@ -556,10 +606,10 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 		   for (j=i=0;auto_D_name[j]&&autotype_dir[i].label;i++){
 		       GtkWidget *a=lookup_widget(popup,auto_D_name[j]);
 		       {
-		           gchar *text,*prg;
+		           gchar *prg;
     		  	   label = gtk_bin_get_child(GTK_BIN(a));
 		           text=g_strconcat(_(autotype_dir[i].label),
-					   " ", FILENAME(en),
+					   " \"", FILENAME(en),"\"",
 					   NULL);
 			   gtk_label_set_text((GtkLabel *)label,
 					   my_utf_string(text));
@@ -578,7 +628,6 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 #if 0
 		    {
         		GtkWidget *label;
-        		gchar *text=NULL;
 			GtkWidget *a=lookup_widget(popup,"autotype_D");
     			label = gtk_bin_get_child(GTK_BIN(a));
 			//label=(GtkLabel *)(((GtkBin *)(a))->child);
