@@ -223,6 +223,7 @@ void on_activate_branch(GtkMenuItem * menuitem, gpointer user_data){
       gtk_tree_view_expand_row(treeview, treepath, FALSE);  
       gdk_flush();
       gtk_tree_view_scroll_to_cell(treeview, treepath, NULL, TRUE, 0.0, 0.0);
+      clear_dnd_selection_list();
       gtk_tree_selection_select_path (selection,treepath);
       gtk_tree_view_set_cursor (treeview,treepath,NULL,FALSE);
       gtk_tree_path_free(treepath);
@@ -393,7 +394,133 @@ autofunction  (GtkTreeView *treeview, tree_entry_t *en,
   return;
 }
 
+void
+on_autotype_C                          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  char *argv[MAX_ARGS];
+  static gchar *command=NULL;
+  int offset=(int)((long)user_data);
+  char *loc;
+  int i=0,j; 
+  gboolean bymime=FALSE;
+  tree_entry_t *en;
+  GtkTreeView *treeview = get_treeview((GtkWidget *) menuitem);
+  GtkTreeIter iter;
+  static gchar *gs=NULL;
 
+  en = get_selected_entry(treeview, &iter);
+  if (gs) {g_free(gs); gs=NULL;}
+
+       
+#ifdef DEBUG
+  printf("DBG:offset=%d\n",offset);
+#endif
+
+  if (!en || !en->path || !IS_PATH(en->type)) return;
+  if (offset >=10) return; /* only ten menu elements */
+  
+  for (i=0;autotype[i].extension;i++){
+       /*printf("dbg: autotype=%s,%s\n",autotype[i].extension,autotype[i].command);*/
+       loc=strstr(en->path,autotype[i].extension);
+       if ((loc)&&(strcmp(loc,autotype[i].extension)==0)) break;
+  }
+  
+  
+  g_free(command);command=NULL;
+  for (j=0; j<=offset && autotype[i+j].command; j++);
+#ifdef DEBUG
+  printf("DBG:command=%s\n",(autotype[i+j].command)?autotype[i+j].command:"NULL"); 
+#endif
+
+  if (offset <= j && autotype[i+offset].command){
+      /* FIXME: do a check whether if the command is sudo allowed
+	       * without password, if so then: */
+      if(getenv("XFFM_USE_SUDO")&&strlen(getenv("XFFM_USE_SUDO"))&&
+	 (strncmp(autotype[i+offset].command,"pkg_add",strlen("pkg_add"))==0 ||
+	  strncmp(autotype[i+offset].command,"pkg_update",strlen("pkg_update"))==0 ||
+	  strncmp(autotype[i+offset].command,"burncd",strlen("burncd"))==0 ||
+	  strncmp(autotype[i+offset].command,"cdrecord",strlen("cdrecord"))==0 ||
+	  strncmp(autotype[i+offset].command,"rpm",strlen("rpm"))==0 ||
+	  strncmp(autotype[i+offset].command,"dpkg",strlen("dpkg"))==0 )
+	) {  
+         if (strlen(autotype[i+offset].command)>249) return;
+	 command=g_strconcat("sudo ",autotype[i+offset].command,NULL);  
+      } else {
+         if (strlen(autotype[i+offset].command)>255) return;
+	 command=g_strdup(autotype[i+offset].command);
+      }  
+  }
+  else {
+	/* must be a mime type */
+	GtkWidget *label;
+	const gchar *text;
+	label = gtk_bin_get_child(GTK_BIN(menuitem));
+	if (!label) return;
+	text=gtk_label_get_text ((GtkLabel *)label);
+	if (!text || strlen(text)==0) return;
+	command=g_strdup(text);
+	/* strip the filename */
+	if (strchr(command,'\"')) *strrchr(command,'\"')=0; /* last " */
+	if (strchr(command,'\"')) *strrchr(command,'\"')=0; /* first " */
+	if (strchr(command,' '))*(strrchr(command,' '))=0;
+#ifdef DEBUG
+  	printf("DBG:textcommand is =%s\n",text);
+#endif
+	bymime=TRUE;
+  }
+#ifdef DEBUG
+  printf("DBG3:command is =%s\n",(command)?command:"NULL");
+#endif
+  if (!strlen(command)) return;
+  
+   
+  
+  /* this function call will enter a gtk loop if querypath != NULL*/ 
+  if (!bymime) {
+	if (!autofunction_workdir (treeview, en,autotype[i+offset].querypath)) {
+	  return;
+	}
+	
+  }
+#ifdef DEBUG
+  printf("DBG4.0:command is =%s\n",(command)?command:"NULL");
+#endif
+  i=parse_runline(argv,command, NULL,NULL);
+    /* command argv done, only need to add the file path */
+  for (j=1;argv[j];j++) {
+	    if (strcmp(argv[j],"\%s")==0){
+		    argv[j]=en->path;
+		    break;
+	    }
+	    if (strstr(argv[j],"\%s")){ /* XXX where is gs defined? are we leaking memory here? */
+		    gs=g_strconcat(argv[j]," ",en->path,NULL);
+		    g_snprintf(gs,strlen(gs),argv[j],en->path);
+		    argv[j]=gs;
+		    break;
+	    }
+    }
+  
+  if (!argv[j]) {
+    argv[i++]=en->path;
+    argv[i] = NULL;
+  } 
+	 
+#ifdef DEBUG
+  printf("DBG:");for (j=0;argv[j];j++) printf("%s ",argv[j]);printf("\n");
+#endif
+
+  if (!bymime && autotype[i+j].command && autotype[i+offset].queued)
+	  autofunction(treeview,en,argv,workdir, autotype[i+offset].queued);
+  else {
+	  runv(treeview,argv);
+  }
+
+  return;
+}
+
+
+#if 0
 void
 on_autotype_C                          (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -403,6 +530,7 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
   int offset=(int)((long)user_data);
   char *loc;
   int i=0,j; 
+  gboolean bymime=FALSE;
   tree_entry_t *en;
     GtkTreeView *treeview = get_treeview((GtkWidget *) menuitem);
     GtkTreeIter iter;
@@ -411,6 +539,7 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
        
   /*printf("DBG:offset=%d\n",offset);*/
   if (!en || !en->path || !IS_PATH(en->type)) return;
+  if (offset >=10) return; /* only ten menu elements */
   
   for (i=0;autotype[i].extension;i++){
        /*printf("dbg: autotype=%s,%s\n",
@@ -418,14 +547,29 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
        loc=strstr(en->path,autotype[i].extension);
        if ((loc)&&(strcmp(loc,autotype[i].extension)==0)) break;
   }
+  
+  command[0]=0;
   for (j=0;j<=offset;j++){
   	/*printf("DBG:command=%s\n",autotype[i+j].command);*/
-  	if (autotype[i+j].command==NULL) return;
-  }
+  	if (autotype[i+j].command==NULL || autotype[i+j].extension==NULL
+			|| !loc || strcmp(loc,autotype[i+j].extension)!=0 ) {
 
-  
+		GtkWidget *label;
+      		const gchar *text;
+    		label = gtk_bin_get_child(GTK_BIN(menuitem));
+		if (!label) return;
+		text=gtk_label_get_text ((GtkLabel *)label);
+		if (!text || strlen(text)==0) return;
+		strncpy(command,text,255);
+  	/*printf("DBG:command is =%s\n",text);*/
+		bymime=TRUE;
+		break;
+		/*return;*/
+	}
+  }
   j=0;
-  if (strstr(autotype[i+offset].command," ")){
+
+  if (command[0]==0 && autotype[i+offset].command) {
       if (strlen(autotype[i+offset].command)>254) return;
       strcpy(command,autotype[i+offset].command);
       /* sudo-izable stuff */
@@ -442,7 +586,11 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
 	      argv[j++]="sudo";
 	      /* when check is implemented, add env var to mcs manager
 	       * (put check also in fstab.c) */
-      }	      
+      }	     
+   } 
+   if (command[0]==0) return;
+   
+   if (strstr(command," ")){
       argv[j++]=strtok(command," ");
       do {
 	      argv[j]=strtok(NULL," ");
@@ -450,22 +598,29 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
 	      j++;
 	      if (j>=64) { argv[63]=0; break; }
       } while (1);
-  } else argv[j++]=autotype[i+offset].command;
-  argv[j++]=en->path;
+   } 
+  
+  if (bymime) argv[j-1]=en->path;
+  else argv[j++]=en->path;
+  
   if (strcmp(argv[0],"burncd")==0 || strcmp(argv[1],"burncd")==0) 
 	  argv[j++]="fixate";
   argv[j]=0;
 
-  /* this function call will enter a gtk loop */ 
-  if (!autofunction_workdir (treeview, en,autotype[i+offset].querypath)) {
+  /* this function call will enter a gtk loop if querypath != NULL*/ 
+  if (!bymime) {
+	if (!autofunction_workdir (treeview, en,autotype[i+offset].querypath)) {
 	  return;
+	}
   }
+  
   /*printf("DBG:");for (j=0;argv[j];j++) printf("%s ",argv[j]);printf("\n");*/
-  if (autotype[i+offset].queued)
+  if (!bymime && autotype[i+j].command && autotype[i+offset].queued)
 	  autofunction(treeview,en,argv,workdir, autotype[i+offset].queued);
   else runv(treeview,argv);
   return;
 }
+#endif
 
 
 /* in this function, the first argument we add will be the output file.
