@@ -131,9 +131,24 @@ GtkTreeView *get_selected_treeview(GtkWidget *w){
 
 /* this function is to workaround a gtk bug in treeviews, where a 
  * while(gtk_events_pending()) will not break... 
- * Let's say 256 iterations suffice... */
-void process_pending_gtk(){
+ * Let's say 256 iterations suffice... 
+ * But not always (if called from a timeout function),
+ * so let's provide a blocking mechanism */
+static gboolean pending_busy=FALSE;
+gboolean processing_pending(void){
+	return pending_busy;
+}
+
+void set_processing_pending(void){
+	pending_busy=TRUE;
+}
+void unset_processing_pending(void){
+	pending_busy=FALSE;
+}
+	
+void process_pending_gtk(void){
   int i;
+  if (processing_pending()) return;
   for (i=0;i<256;i++){
 	  if (!gtk_events_pending()) break;
 	  else gtk_main_iteration();
@@ -490,6 +505,7 @@ char *abreviate(char *path)
     int i;
     static char *label = NULL;
 
+    if (!path) return "";
     if(strlen(path) <= 21)
 	return path;
 
@@ -513,29 +529,34 @@ static char *plain_sizetag(long long tama, long long tamal)
     char *tag = "KB";
     char *tagl = "KB";
     static char buf[64];
+    unsigned long long utama=(unsigned long long) tama;
+    unsigned long long utamal=(unsigned long long) tamal;
 
     buf[0] = 0;
-    if(tama >= (unsigned long long)1024 * 1024)
+    if(utama >= 1024 * 1024)
     {
-	(unsigned long long)tama /= (unsigned long long)1024 *1024;
+	utama /= (unsigned long long)1024 *1024;
 	tag = "GB";
     }
     else if(tama >= 1024)
     {
-	(unsigned long long)tama /= 1024;
+	utama /= 1024;
 	tag = "MB";
     }
 
-    if(tamal >= (unsigned long long)1024 * 1024)
+    if(utamal >= 1024 * 1024)
     {
-	(unsigned long long)tamal /= (unsigned long long)1024 *1024;
+	utamal /= (unsigned long long)1024 *1024;
 	tagl = "GB";
     }
     else if(tamal >= 1024)
     {
-	(unsigned long long)tamal /= 1024;
+	utamal /= 1024;
 	tagl = "MB";
     }
+	tama=utama;
+	tamal=utamal;
+
     sprintf(buf, "%llu %s / %llu %s", tama, tag, tamal, tagl);
 
     return buf;
@@ -631,8 +652,10 @@ void set_font(GtkTreeView * treeview, GtkTreeIter * iterator)
 #if 10
     PangoFontDescription *desc;
     int size;
+    tree_details_t *tree_details = get_tree_details(treeview);
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
     tree_entry_t *en = get_entry(treeview, iterator);
+    /*if (!(tree_details->preferences & ENABLE_RESIZE_FONTS)) return;*/
 
 #if 0
     /* this does not work, probably because there is a theme running...*/
@@ -1087,6 +1110,26 @@ void hide_stop(GtkWidget * widget)
 /*XXX put these separate lib */
 
 #define MAX_COMBO_ELEMENTS 13
+static gint translate_key(gint x){
+	switch (x) {
+		case GDK_KP_Divide: return GDK_slash;
+		case GDK_KP_Subtract: return GDK_minus;
+		case GDK_KP_Multiply: return GDK_asterisk;
+		case GDK_KP_Add: return GDK_plus;
+		case GDK_KP_Space: return GDK_space;
+		case GDK_KP_0: return GDK_0;
+		case GDK_KP_1: return GDK_1;
+		case GDK_KP_2: return GDK_2;
+		case GDK_KP_3: return GDK_3;
+		case GDK_KP_4: return GDK_4;
+		case GDK_KP_5: return GDK_5;
+		case GDK_KP_6: return GDK_6;
+		case GDK_KP_7: return GDK_7;
+		case GDK_KP_8: return GDK_8;
+		case GDK_KP_9: return GDK_9;
+	}
+	return x;
+}
 
 static void clean_history_list(GList **list){
 	GList *tmp;
@@ -1297,7 +1340,7 @@ gint on_key_press_history(GtkWidget * entry, GdkEventKey * event, gpointer data)
 	    return TRUE;
 	  }
 	} else {
-	  *c=event->keyval;	
+	  *c=translate_key(event->keyval);	
     	  if (preselection) {
 	        gtk_editable_delete_text (editable,pos1,-1);
       	        text[0]=gtk_editable_get_chars (editable,0,-1);
@@ -1387,6 +1430,7 @@ gint on_key_press_history(GtkWidget * entry, GdkEventKey * event, gpointer data)
 
 #define HISTORY_ITEMS 13
 static GList **the_list;
+static time_t last_hit;
 
 typedef struct history_dbh_t{
 	int hits;
@@ -1398,11 +1442,29 @@ typedef struct history_dbh_t{
 static int history_compare (gconstpointer a,gconstpointer b){
 	history_dbh_t *da=(history_dbh_t *)a;
 	history_dbh_t *db=(history_dbh_t *)b;
-	if (db->hits != da->hits) return db->hits - da->hits;
-	if (db->last_hit != da->last_hit) return db->last_hit - da->last_hit;
+	/*if (db->last_hit >= last_hit) last_hit = db->last_hit;
+	if (da->last_hit >= last_hit) last_hit = da->last_hit;*/
+	       	
+	if (db->last_hit >= last_hit && da->last_hit < last_hit){
+		return 1;
+	}
+	if (da->last_hit >= last_hit && db->last_hit < last_hit){
+		return -1;
+	}
+	if (db->hits != da->hits) return (db->hits - da->hits);
+	/*if (db->last_hit != da->last_hit) return db->last_hit - da->last_hit;*/
 	return (strcmp(da->path,db->path));
 }
 
+static void history_lasthit(DBHashTable * d)
+{
+    history_dbh_t *history_mem = (history_dbh_t *)DBH_DATA(d);
+    if (!history_mem) g_assert_not_reached();
+    if (history_mem->last_hit >= last_hit) {
+		last_hit=history_mem->last_hit;
+    }
+}
+
 static void history_mklist(DBHashTable * d)
 {
     history_dbh_t *history_mem = (history_dbh_t *)malloc(sizeof(history_dbh_t));
@@ -1428,8 +1490,10 @@ void get_history_list(GList **in_list,char *dbh_file,char *top){
    the_list=in_list;
 
 	   
-   clean_history_list(the_list);	
+   clean_history_list(the_list);
+   last_hit=0;   
    if ((d=DBH_open(dbh_file))!=NULL){
+	DBH_foreach_sweep(d,history_lasthit);	
 	DBH_foreach_sweep(d,history_mklist);	
 	DBH_close(d);
    }
@@ -1489,6 +1553,24 @@ void save_to_history(char *dbh_file,char *path2save){
     	DBH_close(d);
 }
 
+gboolean is_in_history(char *dbh_file,char *path2save){
+    	GString *gs;
+	DBHashTable *d;
+	history_dbh_t *history_dbh;
+	gboolean found=FALSE;
+	
+	if (strlen(path2save) > 255) return;
+	if ((d=DBH_open(dbh_file))==NULL) return FALSE;
+	gs = g_string_new(path2save);
+    	sprintf((char *)DBH_KEY(d), "%10u", g_string_hash(gs));
+   	g_string_free(gs, TRUE);
+
+	history_dbh = (history_dbh_t *)DBH_DATA(d);
+	if (DBH_load(d)) found=TRUE;
+    	DBH_close(d);
+	return found;
+}
+
 void remove_from_history(char *dbh_file,char *path2save){
     	GString *gs;
 	DBHashTable *d;
